/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.api;

public interface ILayerConfiguration {

  /**
   * Create and return an instance of a ILayerConfiguration.
   *
   * @param network the "holding" network for the instance
   * @return the new layer instance
   */
  ILayer instantiate(IModel network);


  /**
   * Defines the valid input type for this ILayer
   *
   * @return InputType
   */
  org.deeplearning4j.nn.conf.inputs.InputType.Type getInputType();


  /**
   * Defines the valid input type for this ILayer
   *
   * @return InputType
   */
  org.deeplearning4j.nn.conf.inputs.InputType.Type getOutputType();


  /**
   * Number of trainable parameter in this layer
   * @return number of parameter
   */
  long numParameters();

  /**
   * An implementation should provide a method to validate the network
   * @return true if no errors found; false otherwise
   */
  boolean isValid();

}
