/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.conf.layer;

import lombok.Builder;
import lombok.experimental.SuperBuilder;
import org.deeplearning4j.nn.conf.layers.LayerValidation;

/**
 * The dense layer is a neural network layer that is connected deeply, which means each neuron in
 * the dense layer receives input from all neurons of its previous layer. The dense layer is found
 * to be the most commonly used layer in the models.
 * <p>
 * In the background, the dense layer performs a matrix-vector multiplication. The values used in
 * the matrix are actually parameters that can be trained and updated with the help of
 * backpropagation.
 * <p>
 * The output generated by the dense layer is an ‘m’ dimensional vector. Thus, dense layer is
 * basically used for changing the dimensions of the vector. Dense layers also applies operations
 * like rotation, scaling, translation on the vector.
 */
@SuperBuilder
public class DenseLayerConfiguration extends FeedForwardLayerConfiguration {

  /**
   * Decides whether we should include a bias vector for calculation purposes or not.
   */
  @Builder.Default
  boolean bias = true;

  

  /**
   * An implementation to validate the network
   *
   * @return true if no errors found; false otherwise
   */
  @Override
  public boolean isValid() {
    LayerValidation.assertNInNOutSet("DenseLayerConfiguration", getName(), -99, getIn(), getOut());
    return super.isValid();
  }
}
