/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.conf.layer;

import lombok.Getter;
import lombok.experimental.SuperBuilder;
import lombok.extern.slf4j.Slf4j;
import net.brutex.ai.dnn.api.ILayer;
import net.brutex.ai.dnn.api.ILayerConfiguration;
import net.brutex.ai.dnn.api.IModel;
import org.deeplearning4j.nn.conf.inputs.InputType;
import org.deeplearning4j.nn.conf.inputs.InputType.Type;

/**
 * A Feed Forward Layer Configuration
 */
@Slf4j
@SuperBuilder
public class FeedForwardLayerConfiguration extends AbstractLayerConfiguration implements ILayerConfiguration {

  @Getter private int in;
  @Getter private int out;

  /**
   * This Fast Forward ILayer will always output data as
   * FF type.
   * @return InputType for FF
   **/
  @Getter
  final InputType.Type outputType = InputType.Type.FF;

  @Getter
  final InputType.Type inputType = InputType.Type.FF;

  /**
   * Create and return an instance of a ILayerConfiguration.
   *
   * @param network the "holding" network for the instance
   * @return the new layer instance
   */
  //@Override
  public ILayer instantiate(IModel network) {
    //Let's do some verifications first
    if(getInputType() != Type.FF) {
      log.error("The {} layer configuration must use an InputType of {}, but found {}",
          this.getClass().getSimpleName(),
          Type.FF.name(),
          getInputType().name());
    }
    return null;
  }

  /**
   * Number of trainable parameter in this layer
   *
   * @return number of parameter
   */
  @Override
  public long numParameters() {
    return in * out + out; //add one extra out for the bias
  }

  /**
   * An implementation should provide a method to validate the network
   *
   * @return true if no errors found; false otherwise
   */
  @Override
  public boolean isValid() {
    boolean result = true;
    if(getInputType() != Type.FF) {
      log.error("The {} layer configuration must use an InputType of {}, but found {}",
          this.getClass().getSimpleName(),
          Type.FF.name(),
          getInputType().name());
      result = false;
    }
    return result;
  }
}
