/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.networks;

import lombok.Getter;
import lombok.Setter;
import net.brutex.ai.dnn.conf.NeuralNetworkConfiguration;
import net.brutex.ai.dnn.api.INeuralNetwork;

/**
 * Artificial Neural Network An artificial neural network (1) takes some input data, and (2)
 * transforms this input data by calculating a weighted sum over the inputs and (3) applies a
 * non-linear function to this transformation to calculate an intermediate state. The three steps
 * above constitute what is known as a layer, and the transformative function is often referred to
 * as a unit. The intermediate states—often termed features—are used as the input into another
 * layer.
 * <p>
 * Through repetition of these steps, the artificial neural network learns multiple layers of
 * non-linear features, which it then combines in a final layer to create a prediction.
 * <p>
 * The neural network learns by generating an error signal that measures the difference between the
 * predictions of the network and the desired values and then using this error signal to change the
 * weights (or parameters) so that predictions get more accurate.
 */
public abstract class ArtificialNeuralNetwork implements INeuralNetwork {

  /**
   * A neural network is created from a configuration.
   * @param conf The (new net.brutex.ai) configuration for the network
   */
  @Getter
  @Setter //TODO make this also final and @NonNull
  private NeuralNetworkConfiguration configuration;
}
