/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.api;

import static net.brutex.ai.dnn.api.dnn.*;
import static org.junit.jupiter.api.Assertions.*;

import java.util.Iterator;
import java.util.concurrent.atomic.AtomicInteger;
import org.apache.commons.lang3.RandomUtils;
import org.deeplearning4j.datasets.iterator.FloatsDataSetIterator;
import org.deeplearning4j.nn.conf.CacheMode;
import org.deeplearning4j.nn.conf.GradientNormalization;
import org.deeplearning4j.nn.conf.Updater;
import org.deeplearning4j.nn.conf.inputs.InputType;
import org.deeplearning4j.nn.conf.layers.ActivationLayer;
import org.deeplearning4j.nn.weights.WeightInitXavier;
import org.junit.jupiter.api.Test;
import org.nd4j.common.primitives.Pair;
import org.nd4j.linalg.activations.impl.ActivationIdentity;
import org.nd4j.linalg.activations.impl.ActivationLReLU;
import org.nd4j.linalg.learning.config.Adam;


class dnnTest {

  @Test
  void testFFLayer() {
    int numFeatures = 128;
    int batchSize = 10;
    int numRows = 1000;
    AtomicInteger cnt = new AtomicInteger(0);
    FloatsDataSetIterator iterator = new FloatsDataSetIterator(floatIterable(numRows, numFeatures), batchSize);

    assertTrue(iterator.hasNext());

    /**
     *     MultiLayerConfiguration confxx = new NeuralNetConfiguration.Builder()
     *         .seed(42)
     *         .updater(UPDATER)
     *         .gradientNormalization(GradientNormalization.RenormalizeL2PerLayer)
     *         .gradientNormalizationThreshold(GRADIENT_THRESHOLD)
     *         .weightInit(WeightInit.XAVIER)
     *         .activation(Activation.IDENTITY)
     *         .list(genLayers())
     *         .setInputType(InputType.convolutional(X_DIM, Y_DIM, CHANNELS))
     *        // .inputPreProcessor("CNN1", new FeedForwardToCnnPreProcessor(Y_DIM, X_DIM, CHANNELS))
     *         .build();
     */

    /**
     *         new DenseLayer.Builder().nIn(INPUT).nOut(X_DIM*Y_DIM*CHANNELS).weightInit(WeightInit.NORMAL).build(),
     *         new ActivationLayer.Builder(new ActivationLReLU(0.2)).build(),
     *         new DenseLayer.Builder().nIn(X_DIM*Y_DIM*CHANNELS).nOut(X_DIM*Y_DIM).build(),
     *         new ActivationLayer.Builder(new ActivationLReLU(0.2)).build(),
     *         new DenseLayer.Builder().nIn(X_DIM*Y_DIM).nOut(X_DIM*Y_DIM).build(),
     *         new ActivationLayer.Builder(new ActivationLReLU(0.2)).build(),
     *         new DenseLayer.Builder().nIn(X_DIM*Y_DIM).nOut(X_DIM*Y_DIM*CHANNELS).activation(Activation.TANH)
     */
    dnn.conf()
        .seed(42)
        .updater( Adam.builder().learningRate(0.0002).beta1(0.5).build() )
        .gradientNormalization( GradientNormalization.RenormalizeL2PerLayer)
        .gradientNormalizationThreshold( 100 )
        .weightInit( new WeightInitXavier() )
        .activation( new ActivationIdentity() )
        .inputType( InputType.convolutional( 28, 28, 1))
        .layer( dnn.DenseLayer(10,30).build() )
        .layer(new ActivationLayer.Builder(new ActivationLReLU(0.2)).build() )

    ;


  }

  protected static Iterable<Pair<float[], float[]>> floatIterable(final int totalRows, final int numColumns) {
    return new Iterable<Pair<float[], float[]>>() {
      @Override
      public Iterator<Pair<float[], float[]>> iterator() {
        return new Iterator<Pair<float[], float[]>>() {
          private final AtomicInteger cnt = new AtomicInteger(0);

          @Override
          public boolean hasNext() {
            return cnt.incrementAndGet() <= totalRows;
          }

          @Override
          public Pair<float[], float[]> next() {
            float[] features = new float[numColumns];
            float[] labels = new float[numColumns];
            for (int i = 0; i < numColumns; i++) {
              features[i] = (float) i;
              labels[i] = RandomUtils.nextFloat(0, 5);
            }
            return Pair.makePair(features, labels);
          }

          @Override
          public void remove() {
            // no-op
          }
        };
      }
    };
  }

}